@@ -50,15 +50,25 @@
 #include <cc++/config.h>
 #endif
 
+#ifndef CCXX_MISSING_H_
+#include <cc++/missing.h>
+#endif
+
 #ifndef	CCXX_THREAD_H_
 #include <cc++/thread.h>
 #endif
 
-#include <iostream>
-#include <fstream>
+#ifndef	CCXX_EXCEPTION_H_
+#include <cc++/exception.h>
+#endif
 
 #ifndef WIN32
+#ifdef	__BORLANDC__
+#include <stdio.h>
+#include <sys/types.h>
+#else
 #include <cstdio>
+#endif
 #include <dirent.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
@@ -66,7 +76,7 @@
 #include <direct.h>
 #endif
 
-#ifdef	HAVE_MACH_DYLD
+#ifdef  HAVE_MACH_DYLD
 #include <mach-o/dyld.h>
 #endif
 
@@ -82,19 +92,29 @@ typedef	unsigned long pos_t;
 #define	caddr_t	char *
 typedef size_t ccxx_size_t;
 #else
+#ifndef	__BORLANDC__
 typedef LONG off_t;
+#endif
 typedef void* caddr_t;
 typedef DWORD ccxx_size_t;
 #endif
 
-class File
+#ifndef	PATH_MAX
+#define	PATH_MAX	256
+#endif
+
+#ifndef	NAME_MAX
+#define	NAME_MAX	64
+#endif
+
+class __EXPORT File
 {
 public:
 	enum Error
 	{
 		errSuccess = 0,
 		errNotOpened,
-		errMmapFailed,
+		errMapFailed,
 		errInitFailed,
 		errOpenDenied,
 		errOpenFailed,
@@ -132,6 +152,7 @@ protected:
 		off_t pos;
 	} fcb_t;
 
+public:
 #ifdef	WIN32
 	enum Open
 	{
@@ -198,9 +219,9 @@ protected:
 
 	enum Mapping
 	{
-		mappedRead, // = FILE_MAPPED_READ
-		mappedWrite, // = FILE_MAPPED_WRITE,
-		mappedReadWrite // = FILE_MAPPED_RDWR
+		mappedRead,
+		mappedWrite,
+		mappedReadWrite
 	};
 #else
 	enum Mapping
@@ -218,6 +239,13 @@ protected:
 #endif
 	typedef enum Complete Complete;
 	typedef enum Mapping Mapping;
+
+public:
+	static const char *getExtension(const char *path);
+	static const char *getFilename(const char *path);
+	static char *getFilename(const char *path, char *buffer, size_t size = NAME_MAX);
+	static char *getDirname(const char *path, char *buffer, size_t size = PATH_MAX);
+	static char *getRealpath(const char *path, char *buffer, size_t size = PATH_MAX);
 };
 
 #ifndef WIN32
@@ -298,22 +326,46 @@ public:
  * @author David Sugar <dyfet@ostel.com>
  * @short low level directory access class.
  */
-class CCXX_CLASS_EXPORT Dir
+class __EXPORT Dir : public File
 {
 private:
 #ifndef WIN32
 	DIR *dir;
+#ifdef	HAVE_READDIR_R
+	struct dirent *save;
+	char save_space[sizeof(struct dirent) + PATH_MAX + 1];
+#endif
+	struct dirent *entry;
 #else
 	HANDLE hDir;
-	WIN32_FIND_DATA data;
+	WIN32_FIND_DATA data, fdata;
 	char *name;
 #endif
 
 public:
-	Dir(const char *name);
+	Dir(const char *name = NULL);
+
+	static bool create(const char *path, Attr attr = attrGroup);
+	static bool remove(const char *path);
+	static bool setPrefix(const char *path);
+	static bool getPrefix(char *path, size_t size = PATH_MAX);
+
+	void open(const char *name);
+	void close(void);
+
 	virtual ~Dir();
 
-	char *getName(void);
+	const char *getName(void);
+
+	const char *operator++()
+		{return getName();};
+
+	const char *operator++(int)
+		{return getName();};
+
+	const char *operator*();
+
+	bool rewind(void);
 
 	bool operator!()
 #ifndef WIN32
@@ -335,7 +387,7 @@ public:
  * @author David Sugar <dyfet@ostel.com>
  * @short Portable random disk file access.
  */
-class CCXX_CLASS_EXPORT RandomFile : protected Mutex, public File
+class __EXPORT RandomFile : protected Mutex, public File
 {
 private:
 	Error errid;
@@ -365,7 +417,7 @@ protected:
 	/**
 	 * Create an unopened random access file.
 	 */
-	RandomFile();
+	RandomFile(const char *name = NULL);
 
 	/**
 	 * Default copy constructor.
@@ -387,8 +439,8 @@ protected:
 	 * @return errExtended.
 	 * @param errstr error string.
 	 */
-	inline Error error(char *errstr)
-		{return error(errExtended, errstr);};
+	inline Error error(char *err)
+		{return error(errExtended, err);};
 
 	/**
 	 * Used to enable or disable throwing of exceptions on
@@ -508,7 +560,7 @@ public:
  * @author David Sugar <dyfet@ostel.com>
  * @short This class defines a database I/O file service that can be shared by multiple threads.
  */
-class CCXX_CLASS_EXPORT ThreadFile : public RandomFile
+class __EXPORT ThreadFile : public RandomFile
 {
 private:
 	ThreadKey state;
@@ -592,7 +644,7 @@ public:
  * @author David Sugar <dyfet@ostel.com>
  * @short This class defines a database I/O file service that can be shared by multiple processes.
  */
-class CCXX_CLASS_EXPORT SharedFile : public RandomFile
+class __EXPORT SharedFile : public RandomFile
 {
 private:
 	fcb_t fcb;
@@ -681,23 +733,26 @@ public:
 	bool operator--(void);
 };
 
-#ifndef WIN32
 /**
  * Create and map a disk file into memory.  This portable class works
  * under both Posix via mmap and under the win32 API. A mapped file
- * can be referenced directly by it's memory segment. One can map 
+ * can be referenced directly by it's memory segment. One can map
  * and unmap portions of a file on demand, and update
  * changed memory pages mapped from files immediately through sync().
- * 
+ *
  * @author David Sugar <dyfet@ostel.com>
  * @short Map a named disk file into memory.
  * @todo implement in win32
  */
-class CCXX_CLASS_EXPORT MappedFile : public RandomFile
+class __EXPORT MappedFile : public RandomFile
 {
 private:
 	fcb_t fcb;
 	int prot;
+#ifdef	WIN32
+	HANDLE map;
+	char mapname[64];
+#endif
 
 public:
 	/**
@@ -713,14 +768,14 @@ public:
 	 * Map a portion or all of a specified file in the specified
 	 * shared memory access mode.  Valid mapping modes include
 	 * mappedRead, mappedWrite, and mappedReadWrite.
-	 * 
+	 *
 	 * @param fname pathname of file to map into memory.
 	 * @param offset from start of file to begin mapping in bytes.
 	 * @param size of mapped area in bytes.
 	 * @param mode to map file.
 	 */
-	MappedFile(const char *fname, pos_t offset, size_t size, Access mode);	
-	
+	MappedFile(const char *fname, pos_t offset, size_t size, Access mode);
+
 	/**
 	 * Release a mapped section of memory associated with a file.  The
 	 * mapped area is updated back to disk.
@@ -733,8 +788,7 @@ public:
 	 * the disk file and wait for completion.  This assures the memory
 	 * mapped from the file is written back.
 	 */
-	inline void sync(void)
-		{msync(fcb.address, fcb.len, MS_SYNC);};
+	void sync(void);
 
 	/**
 	 * Synchronize a segment of memory mapped from a segment fetch.
@@ -748,7 +802,7 @@ public:
 	 * Map a portion of the memory mapped from the file back to the
 	 * file and do not wait for completion.  This is useful when mapping
 	 * a database file and updating a single record.
-	 * 
+	 *
 	 * @param offset offset into the mapped region of memory.
 	 * @param len length of partial region (example, record length).
 	 */
@@ -776,7 +830,7 @@ public:
 	 * of the disk file.  This really is used for convience of matching
 	 * operations between Update and Fetch, as one could simply have
 	 * accessed the base pointer where the file was mapped directly.
-	 * 
+	 *
 	 * @param offset from start of mapped memory.
 	 */
 	inline caddr_t fetch(size_t offset = 0)
@@ -792,158 +846,7 @@ public:
 	 */
 	caddr_t fetch(off_t pos, size_t len);
 };
-#endif // !WIN32
 
-/**
- * The Pipe uses system kernel buffering to hold data being passed either
- * between two execution contexts within the same process, or between
- * different processes.  Unlike thread's "Buffer", Pipe uses system
- * descriptors and kernel memory.  Under Posix, the size of the pipe and
- * associated kernel memory is always a fixed constant as defined by
- * _PC_PIPE_BUF.  The Common C++ "pipe" class primarily deals with "atomic"
- * transfers of fixed sized objects through pipes.  Pipes may pass
- * data arbitrarily and can also be used through the "pipestream" class.
- *
- * The "Pipe" class is not meant to be a true "public" class, but as a
- * builder class for deriving other classes.
- *
- * @author David Sugar <dyfet@ostel.com>
- * @short kernel buffering between processes and/or threads.
- */
-class CCXX_CLASS_EXPORT Pipe
-{
-protected:
-#ifndef WIN32
-	int fd[2];
-#else
-	HANDLE fd[2];
-#endif
-	int objcount;
-	int objsize;
-
-	/**
-	 * Get the object size for atomic operations.
-	 *
-	 * @return size of atomic operations.
-	 */
-	inline int getSize(void)
-		{return objsize;};
-
-	/**
-	 * Sender is often used for implementing a fork()'d message port
-	 * between processes.  By stopping the sender, the current pipe
-	 * can then only be used for receiving, and the sender is presumed
-	 * to be in the other half of a fork()'d process.
-	 * 
-	 * @see #endReceiver
-	 */
-	inline void endSender(void)
-#ifndef WIN32
-		{close(fd[1]);};
-#else
-		{CloseHandle(fd[1]); fd[1] = NULL; };
-#endif
-	/**
-	 * Receiver is often used for implementing a fork()'d message port
-	 * between processes.  By stopping the receiver, the current pipe
-	 * can then only be used for sending, and the receiver is presumed
-	 * to be in the other half of a fork()'d process.
-	 *
-	 * @see #endSender
-	 */
-	inline void endReceiver(void)
-#ifndef WIN32
-		{close(fd[0]);};
-#else
-		{CloseHandle(fd[0]); fd[0] = NULL; };
-#endif
-
-	/**
-	 * Create a kernel pipe descriptor set using pipe().  On systems
-	 * which allow the pipe kernel buffer to be defined, a size for
-	 * aligned atomic transfers can be set, as well as the number of
-	 * atomic objects the kernel will buffer.  On Posix systems, these
-	 * options are ignored.
-	 *
-	 * @param size objsize of atomic objects to send.
-	 * @param count count of atomic objects to kernel buffer.
-	 */
-	Pipe(int size = 512, int count = 1);
-
-	/**
-	 * Destroy the pipe and kernel descriptor resources.
-	 */
-	virtual ~Pipe();
-
-	/**
-	 * Create a pipe as a duplicate of an existing pipe.
-	 * 
-	 * @param orig pipe to duplicate.
-	 */
-	Pipe(const Pipe &orig);
-
-	Pipe &operator=(const Pipe &orig);
-
-	/* depreciated methods */
-	void sender(void)
-		{endReceiver();};
-
-	void receiver(void)
-		{endSender();};
-
-	int read(void *buf, int len)
-#ifndef WIN32
-		{return ::read(fd[0], (char *)buf, len);};
-#else
-		{DWORD dwLen; return ReadFile(fd[0],buf,len,&dwLen,NULL) ? dwLen : (DWORD)-1; };
-#endif
-
-	int write(void *buf, int len)
-#ifndef WIN32
-		{return ::write(fd[1], (char *)buf, len);};
-#else
-		{DWORD dwLen; return WriteFile(fd[1],buf,len,&dwLen,NULL) ? dwLen : (DWORD)-1; };
-#endif
-
-public:
-	/**
-	 * Used to see if the pipe has any open entities.
-	 */
-	bool operator!();
-
-	/**
-	 * Perform an object atomic transfer of data from a pipe.
-	 * 
-	 * @return number of bytes actually read if successful.
-	 * @param addr pointer to store read data.
-	 */
-#ifndef WIN32
-	inline int receive(void *addr)
-		{return ::read(fd[0], (char *)addr, objsize);};
-#else
-	int receive(void *addr);
-#endif
-
-	/**
-	 * Perform an object atomic transfer of data to a pipe.
-	 * 
-	 * @return number of bytes written if successful.
-	 * @param addr pointer to write data from.
-	 */
-#ifndef WIN32
-	inline int send(void *addr)
-		{return ::write(fd[1], (char *)addr, objsize);};
-#else
-	int send(void *addr);
-#endif
-
-	/**
-	 * Verify this object is "valid".
-	 *
-	 * @return true if valid.
-	 */
-	bool isValid(void);
-};
 
 /**
  * The DSO dynamic loader class is used to load object files.  On
@@ -953,26 +856,26 @@ public:
  * @author David Sugar <dyfet@ostel.com>
  * @short Dynamic class file loader.
  */
-class DSO
+class __EXPORT DSO
 {
 private:
+	const char *err;
 #ifdef	HAVE_MODULES
 	static Mutex mutex;
-	static DSO *first, *last;
+	static DSO *first;
+	static DSO *last;
 	DSO *next, *prev;
+	const char *id;
 #if	defined(HAVE_MACH_DYLD)
-	NSModule oModule;
-	char *err;	
-#elif	defined(HAVE_SHL_LOAD)
-	char *err;
-	shl_t image;
+        NSModule oModule;
+#elif   defined(HAVE_SHL_LOAD)
+        shl_t image;
 #elif	defined(WIN32)
-	char *err;
 	HINSTANCE hImage;
 #else
 	void *image;
 #endif
-	CCXX_MEMBER_EXPORT(void) loader(const char *filename, bool resolve);
+	void loader(const char *filename, bool resolve);
 #endif
 
 public:
@@ -998,93 +901,101 @@ public:
 	 * Retrieve error indicator associated with DSO failure.  This
 	 * is often used in catch handlers.
 	 */
-	char *getError(void);
+	inline const char *getError(void)
+		{return err;};
 
 	/**
 	 * Detach a DSO object from running memory.
 	 */
 #ifdef	HAVE_MODULES
-	CCXX_MEMBER_EXPORT(virtual) ~DSO();
+	virtual ~DSO();
 #endif
 
 	/**
 	 * Lookup a symbol in the loaded file.
 	 */
 #ifdef	HAVE_MODULES
-	CCXX_MEMBER_EXPORT(void*) operator[](const char *sym);
+	void* operator[](const char *sym);
 #else
 	void *operator[](const char *)
 		{return NULL;};
 #endif
 
 #ifdef	HAVE_MODULES
-	static CCXX_EXPORT(void) dynunload(void);
+	static void dynunload(void);
 #else
 	static void dynunload(void)
 		{return;};
 #endif
 
+	/**
+	 * Find a specific DSO object by filename.
+	 *
+	 * @param name of DSO object file (partial).
+	 */
+	static DSO *getObject(const char *name);
+
 	/**
 	 * See if DSO object is valid.
 	 *
 	 * @return true if valid.
 	 */
-	CCXX_MEMBER_EXPORT(bool) isValid(void);
+	bool isValid(void);
 };
 
 /** @relates RandomFile */
-bool isDir(const char *path);
+bool __EXPORT isDir(const char *path);
 /** @relates RandomFile */
-bool isFile(const char *path);
+bool __EXPORT isFile(const char *path);
 #ifndef WIN32
 /** @relates RandomFile */
-bool isDevice(const char *path);
+bool __EXPORT isDevice(const char *path);
 #else
 /** @relates RandomFile */
 inline bool isDevice(const char *path)
 { return false; }
 #endif
 /** @relates RandomFile */
-bool canAccess(const char *path);
+bool __EXPORT canAccess(const char *path);
 /** @relates RandomFile */
-bool canModify(const char *path);
+bool __EXPORT canModify(const char *path);
 
 #ifdef	COMMON_STD_EXCEPTION
 
 class DirException : public IOException
 {
 public:
-	DirException(std::string str) : IOException(str) {};
+	DirException(const String &str) : IOException(str) {};
 };
 
-class DSOException : public IOException
+class __EXPORT DSOException : public IOException
 {
 public:
-	DSOException(std::string str) : IOException(str) {};
+	DSOException(const String &str) : IOException(str) {};
 };
 
-class FIFOException : public IOException
+class __EXPORT FIFOException : public IOException
 {
 public:
-	FIFOException(std::string str) : IOException(str) {};
+	FIFOException(const String &str) : IOException(str) {};
 };
 
-class PipeException : public IOException
+class __EXPORT PipeException : public IOException
 {
 public:
-	PipeException(std::string str) : IOException(str) {};
+	PipeException(const String &str) : IOException(str) {};
 };
 
-class FileException : public IOException
+class __EXPORT FileException : public IOException
 {
 public:
-	FileException(std::string str) : IOException(str) {};
+	FileException(const String &str) : IOException(str) {};
 };
 
 #endif
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 #endif
