@@ -46,6 +46,14 @@
 #ifndef	CCXX_PERSIST_H_
 #define	CCXX_PERSIST_H_
 
+#ifndef CCXX_CONFIG_H_
+#include <cc++/config.h>
+#endif
+
+#ifndef CCXX_EXCEPTIONS_H_
+#include <cc++/exception.h>
+#endif
+
 #ifndef CCXX_MISSING_H_
 #include <cc++/missing.h>
 #endif
@@ -280,6 +288,8 @@ public:
 	 * This Flushes the buffers and closes the Persistence::Engine
 	 * this must happen before the underlying stream is shut down
 	 */
+  void sync();
+  
 	virtual ~Engine();
 
 
@@ -312,7 +322,9 @@ public:
 	void write(float i)  THROWS (Exception) { CCXX_ENGINEWRITE_REF(i); }
 	void write(double i) THROWS (Exception) { CCXX_ENGINEWRITE_REF(i); }
 #undef CCXX_ENGINEWRITE_REF
+
 	void write(const String& str) THROWS (Exception);
+	void write(const std::string& str) THROWS (Exception);
 
 	// Every write operation boils down to one or more of these
 	void writeBinary(const uint8* data, const uint32 size) THROWS (Exception);
@@ -348,6 +360,7 @@ public:
 #undef CCXX_ENGINEREAD_REF
 
 	void read(String& str) THROWS (Exception);
+	void read(std::string& str) THROWS (Exception);
 
 	// Every read operation boild down to one or more of these
 	void readBinary(uint8* data, uint32 size) THROWS (Exception);
@@ -464,6 +477,11 @@ __EXPORT Engine& operator >>( Engine& ar, String& ob) THROWS (Engine::Exception)
 /** @relates Engine */
 __EXPORT Engine& operator <<( Engine& ar, String ob)  THROWS (Engine::Exception);
 
+/** @relates Engine */
+__EXPORT Engine& operator >>( Engine& ar, std::string& ob) THROWS (Engine::Exception);
+/** @relates Engine */
+__EXPORT Engine& operator <<( Engine& ar, std::string ob)  THROWS (Engine::Exception);
+
 /** @relates Engine */
 __EXPORT Engine& operator >>( Engine& ar, bool& ob) THROWS (Engine::Exception);
 /** @relates Engine */
@@ -585,7 +603,3 @@ Engine& operator >>( Engine& ar, typename std::map<Key,Value>& ob) THROWS (Engin
  * c-basic-offset: 8
  * End:
  */
-
-
-
-
