@@ -68,7 +68,7 @@ namespace ost {
  * @short Peocess wrapper class.
  * @author David Sugar <dyfet@ostel.com>
  */
-class CCXX_CLASS_EXPORT Process
+class __EXPORT Process
 {
 public:
 #ifndef	WIN32
@@ -80,6 +80,15 @@ public:
 	 */
 	static void detach(void);
 
+	/**
+	 * Attach the current process to another device
+	 * or i/o session.  It is deamonified and dissasociated
+	 * with the prior parent process and controlling terminal.
+	 *
+	 * @param devname path to attach to.
+	 */
+	static void attach(const char *devname);
+
 	/**
 	 * Set a posix compliant signal handler.
 	 *
@@ -113,8 +122,27 @@ public:
 	 * @return error code from process.
 	 * @param exec name of executable.
 	 * @param argv list of command arguments.
+	 * @param wait for process to exit before return.
 	 */
-	static int spawn(const char *exec, const char **argv);
+	static int spawn(const char *exec, const char **argv, bool wait = true);
+
+	/**
+	 * Get the exit status of another process, waiting for it
+	 * to exit.
+	 *
+	 * @return exit code from process.
+	 * @param pid process id.
+	 */
+	static int join(int pid);
+
+	/**
+	 * Cancel a running child process.
+	 *
+	 * @return 0 on success.
+	 * @param pid process id.
+	 * @param sig cancel signal to apply.
+	 */
+	static bool cancel(int pid, int sig = 0);
 
 	/**
 	 * Get system environment.
@@ -132,10 +160,46 @@ public:
 	 * @param overwrite true if replace existing symbol.
 	 */
 	static void setEnv(const char *name, const char *value, bool overwrite);
+
+	/**
+	 * Get etc prefix path.
+	 *
+	 * @return etc prefix.
+	 */
+	static const char *getConfigDir(void)
+		{return ETC_PREFIX;};
+
+	/**
+	 * Get home directory.
+	 *
+	 * @return user home directory.
+	 */
+	static const char *getHomeDir(void);
+
+	/**
+	 * Get user name.
+	 *
+	 * @return user login id.
+	 */
+	static const char *getUser(void);
+
+	/**
+	 * Set user id by name.
+	 *
+	 * @return true if successful.
+	 */
+	static bool setUser(const char *id, bool grp = true);
+
+	/**
+	 * Set the effective group id by name.
+	 *
+	 * @return true if successful.
+	 */
+	static bool setGroup(const char *id);
 };
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 #endif
