@@ -142,7 +142,9 @@ public:
 		errMulticastDisabled,
 		errTimeout,
 		errNoDelay,
-		errExtended
+		errExtended,
+		errLookupFail,
+		errSearchErr
 	};
 
 	typedef enum Error Error;
@@ -461,7 +463,43 @@ public:
 #ifdef	CCXX_IPV6
 	IPV6Host getIPV6Local(tpport_t *port = NULL) const;
 #endif
-	
+
+	/**
+	 * Perform NAT table lookup for this socket.
+	 * Used to allow an application to know the original ip:port 
+	 * pair the the client "thinks" it is connecting to. Used mostly 
+	 * to transparently impersonate a remote server/service.
+	 *
+	 * On error, 0.0.0.0:0 is returned and one of the following error codes 
+	 * is set: errServiceUnavailable - if nat is not supported on the 
+	 * current platform or if it was not compiled; errLookupFail - if the
+	 * nat syscall failed for some reason (extended error code); 
+	 * errSearchErr - if the socket does not have nat information (i.e.
+	 * is not nated).
+	 * 
+	 * NAT lookup is supported on NetFilter for ipv4 and ipv6 (Linux), 
+	 * IPFilter for ipv4 (Solaris, *BSD except OpenBSD, HP-UX, etc.) and
+	 * Packet Filter for ipv4 and ipv6 (OpenBSD).
+	 * When using IPFilter or Packet Filter, the first NAT lookup must be 
+	 * performed as root (the NAT device is read only for root and is opened
+	 * once, unless an error occurs). Permissions on the nat device may be
+	 * changed to solve this.
+	 *
+	 * \warning When using IPFilter and Packet Filter, application data model
+	 * must be the same as the running kernel (32/64 bits).
+	 *
+	 * @param port ptr to NATed port number on local host. 
+	 * @return NATed host address that this socket is related to.
+	 */
+	IPV4Host getIPV4NAT(tpport_t *port = NULL) const;
+
+	inline IPV4Host getNAT(tpport_t *port) const
+		{return getIPV4NAT(port);}
+
+#ifdef	CCXX_IPV6
+	IPV6Host getIPV6NAT(tpport_t *port = NULL) const;
+#endif
+
 	/**
 	 * Used to specify blocking mode for the socket.  A socket
 	 * can be made non-blocking by setting setCompletion(false)
