@@ -235,8 +235,7 @@ public:
 	 *
 	 * @param string to set with.
 	 */
-	inline String(std::string str)
-		{String(str.c_str());};
+	String(std::string str);
 
 	/**
 	 * Create a new string from a subset of another string.	
