@@ -128,6 +128,7 @@ public:
 
 private:
 	const char *agent, *referer, *cookie, *pragma, *user, *password;
+	const char *proxyUser, *proxyPasswd;
 	const char *localif;
 	IPV4Host proxyHost;
 #ifdef	CCXX_IPV6
@@ -138,6 +139,7 @@ private:
 	Encoding encoding;
 	Protocol protocol;
 	Authentication auth;
+	Authentication proxyAuth;
 	timeout_t timeout;
 	bool persistent;
 	bool follow;
@@ -147,6 +149,8 @@ private:
 	URLStream(const URLStream& rhs);
 
 protected:
+	ost::String m_host, m_address;
+
 	Error sendHTTPHeader(const char *url, const char **vars, int bufsize);
 	int underflow(void);
 
@@ -207,6 +211,16 @@ public:
 	 */
 	Error get(const char *url, int buffer = 512);
 
+	/**
+	* Get URL data from a named stream of a known buffer size.
+	* Requesting URL defined in previous calls of setAddress() and 
+	* setHost() functions.
+	*
+	* @return url error code.
+	* @param buffer size of buffer.
+	*/
+	Error get(int buffer = 512);		
+
 	/**
 	 * Submit URL with vars passed as argument array.  This submit
 	 * assumes "GET" method.  Use "post" member to perform post.
@@ -249,6 +263,22 @@ public:
 	 */
 	void setReferer(const char *str);
 
+	/**
+	* Set the host for the url
+	*
+	* @param str host address.
+	*/
+	inline void setHost(const char *str)
+	{m_host = str;};
+
+	/**
+	* Set the address for the url
+	*
+	* @param str address in the URL.
+	*/
+	inline void setAddress(const char *str)
+	{m_address = str;};
+
 	/**
 	 * Set the cookie to pass.
 	 *
@@ -281,6 +311,30 @@ public:
 	 */
 	void setAuthentication(Authentication a, const char *str = NULL);
 
+ 	/**
+ 	* Set proxy user id for the url.
+ 	*
+ 	* @param str user id.
+ 	*/
+ 	inline void setProxyUser(const char *str)
+ 	{proxyUser = str;};
+ 
+ 	/**
+ 	* Set proxy password for the url.
+ 	*
+ 	* @param str password.
+ 	*/
+ 	inline void setProxyPassword(const char *str)
+ 	{proxyPasswd = str;};
+ 
+ 	/**
+ 	* Set proxy authentication type for the url.
+ 	*
+ 	* @param a authentication.
+ 	* @param str string.
+ 	*/
+ 	void setProxyAuthentication(Authentication a, const char *str = NULL);
+
 	/**
 	 * Set the pragmas.
 	 *
