@@ -40,10 +40,18 @@
 
 #include <cc++/config.h>
 #include <cc++/export.h>
+#include <cc++/address.h>
 #include <cc++/socket.h>
 #include "private.h"
 #ifndef WIN32
 #include <cerrno>
+#define socket_errno errno
+#else
+#define socket_errno WSAGetLastError()
+#endif
+
+#ifndef	INADDR_LOOPBACK
+#define	INADDR_LOOPBACK	(unsigned long)0x7f000001
 #endif
 
 #ifdef	CCXX_NAMESPACES
@@ -54,12 +62,12 @@ namespace ost {
 TimerPort::TimerPort()
 {
 	active = false;
-	gettimeofday(&timer, NULL);
+       SysTime::getTimeOfDay(&timer);
 }
 
 void TimerPort::setTimer(timeout_t timeout)
 {
-	gettimeofday(&timer, NULL);
+       SysTime::getTimeOfDay(&timer);
 	active = false;
 	if(timeout)
 		incTimer(timeout);
@@ -93,7 +101,7 @@ timeout_t TimerPort::getTimer(void) const
 	if(!active)
 		return TIMEOUT_INF;
 
-	gettimeofday(&now, NULL);
+       SysTime::getTimeOfDay(&now);
 	diff = (timer.tv_sec - now.tv_sec) * 1000l;
 	diff += (timer.tv_usec - now.tv_usec) / 1000l;
 
@@ -111,7 +119,7 @@ timeout_t TimerPort::getElapsed(void) const
 	if(!active)
 		return TIMEOUT_INF;
 
-	gettimeofday(&now, NULL);
+       SysTime::getTimeOfDay(&now);
 	diff = (now.tv_sec -timer.tv_sec) * 1000l;
 	diff += (now.tv_usec - timer.tv_usec) / 1000l;
 	if(diff < 0)
@@ -180,11 +188,12 @@ timeout_t TimerPort::getElapsed(void) const
 #endif
 
 SocketPort::SocketPort(SocketService *svc, TCPSocket &tcp) :
-	Socket(accept(tcp.so, NULL, NULL)),
-	detect_pending(true),
-	detect_output(false),
-	detect_disconnect(true)
+Socket(accept(tcp.so, NULL, NULL))
 {
+	detect_pending = true;
+	detect_output = false;
+	detect_disconnect = true;
+
 #ifdef WIN32
 	// FIXME: error handling
 	event = CreateEvent(NULL,TRUE,FALSE,NULL);
@@ -200,13 +209,41 @@ SocketPort::SocketPort(SocketService *svc, TCPSocket &tcp) :
 #endif
 	{
 		setError(false);
-		if(svc)
-			svc->attach(this);
+    if( svc )
+  		svc->attach(this);
 	}
 }
 
-SocketPort::SocketPort(SocketService *svc, const InetAddress &ia, 
-		       tpport_t port) :
+#ifdef  CCXX_IPV6
+SocketPort::SocketPort(SocketService *svc, TCPV6Socket &tcp) :
+Socket(accept(tcp.so, NULL, NULL))
+{
+	detect_pending = true;
+	detect_output = false;
+	detect_disconnect = true;
+
+#ifdef WIN32
+	// FIXME: error handling
+	event = CreateEvent(NULL,TRUE,FALSE,NULL);
+#endif
+	next = prev = NULL;
+	service = NULL;
+
+	// FIXME: use macro here and in other files...
+#ifndef WIN32
+	if(so > -1)
+#else
+	if(so != INVALID_SOCKET)
+#endif
+	{
+		setError(false);
+    if( svc )
+  		svc->attach(this);
+	}
+}
+#endif
+
+SocketPort::SocketPort(SocketService *svc, const IPV4Address &ia, tpport_t port) :
 Socket(AF_INET, SOCK_DGRAM, 0)
 {
 #ifdef WIN32
@@ -221,20 +258,58 @@ Socket(AF_INET, SOCK_DGRAM, 0)
 	addr.sin_family = AF_INET;
 	addr.sin_addr = getaddress(ia);
 	addr.sin_port = htons(port);
+	detect_pending = true;
+	detect_output = false;
+	detect_disconnect = true;
+
 	if(bind(so, (struct sockaddr *)&addr, (socklen_t)sizeof(addr)))
 	{
 		endSocket();
-		error(errBindingFailed);
+		error(errBindingFailed,"Could not bind socket",socket_errno);
 		return;
 	}
 	state = BOUND;
 	setError(false);
-	if(svc)
-		svc->attach(this);
+
+  if(svc)
+  	svc->attach(this);
 }
 
-SocketPort::SocketPort(SocketService *svc, const InetHostAddress &ih, 
-		       tpport_t port) :
+#ifdef  CCXX_IPV6
+SocketPort::SocketPort(SocketService *svc, const IPV6Address &ia, tpport_t port) :
+Socket(AF_INET6, SOCK_DGRAM, 0)
+{
+#ifdef WIN32
+	// FIXME: error handling
+	event = CreateEvent(NULL,TRUE,FALSE,NULL);
+#endif
+	struct sockaddr_in6 addr;
+
+	memset(&addr, 0, sizeof(addr));
+	next = prev = NULL;
+	service = NULL;
+	addr.sin6_family = AF_INET6;
+	addr.sin6_addr = getaddress(ia);
+	addr.sin6_port = htons(port);
+	detect_pending = true;
+	detect_output = false;
+	detect_disconnect = true;
+
+	if(bind(so, (struct sockaddr *)&addr, (socklen_t)sizeof(addr)))
+	{
+		endSocket();
+		error(errBindingFailed,"Could not bind socket",socket_errno);
+		return;
+	}
+	state = BOUND;
+	setError(false);
+
+  if(svc)
+  	svc->attach(this);
+}
+#endif
+
+SocketPort::SocketPort(SocketService *svc, const IPV4Host &ih, tpport_t port) :
 Socket(AF_INET, SOCK_STREAM, 0)
 {
 #ifdef WIN32
@@ -249,6 +324,8 @@ Socket(AF_INET, SOCK_STREAM, 0)
 	addr.sin_family = AF_INET;
 	addr.sin_addr = getaddress(ih);
 	addr.sin_port = htons(port);
+	detect_pending = true;
+	detect_disconnect = true;
 
 #ifndef WIN32
 	long opts = fcntl(so, F_GETFL);
@@ -290,12 +367,84 @@ Socket(AF_INET, SOCK_STREAM, 0)
 #endif
 
 	setError(false);
+	detect_output = (state == CONNECTING);
+
 	if(svc)
-		svc->attach(this);
-        if(state == CONNECTING)
-                setDetectOutput(true);
+  		svc->attach(this);
+
+//	if(state == CONNECTING)
+//		setDetectOutput(true);
 }
 
+#ifdef  CCXX_IPV6
+SocketPort::SocketPort(SocketService *svc, const IPV6Host &ih, tpport_t port) :
+Socket(AF_INET6, SOCK_STREAM, 0)
+{
+#ifdef WIN32
+	// FIXME: error handling
+	event = CreateEvent(NULL,TRUE,FALSE,NULL);
+#endif
+	struct sockaddr_in6 addr;
+
+	memset(&addr, 0, sizeof(addr));
+	next = prev = NULL;
+	service = NULL;
+	addr.sin6_family = AF_INET6;
+	addr.sin6_addr = getaddress(ih);
+	addr.sin6_port = htons(port);
+	detect_pending = true;
+	detect_disconnect = true;
+
+#ifndef WIN32
+	long opts = fcntl(so, F_GETFL);
+	fcntl(so, F_SETFL, opts | O_NDELAY);
+#else
+	u_long opts = 1;
+	ioctlsocket(so,FIONBIO,&opts);
+#endif
+
+	int rtn = ::connect(so, (struct sockaddr *)&addr, (socklen_t)sizeof(addr));
+
+	if(!rtn)
+	{
+		state = CONNECTED;
+	}
+	else
+	{
+#ifndef WIN32
+		if(errno == EINPROGRESS)
+#else
+		if(WSAGetLastError() == WSAEINPROGRESS || WSAGetLastError() == WSAEWOULDBLOCK)
+#endif
+		{
+			state = CONNECTING;
+		}
+		else
+		{
+			endSocket();
+			connectError();
+			return;
+		}
+	}
+
+#ifndef WIN32
+	fcntl(so, F_SETFL, opts);
+#else
+	opts = 0;
+	ioctlsocket(so,FIONBIO,&opts);
+#endif
+
+	setError(false);
+	detect_output = (state == CONNECTING);
+
+	if(svc)
+  		svc->attach(this);
+
+//	if(state == CONNECTING)
+//		setDetectOutput(true);
+}
+#endif
+
 SocketPort::~SocketPort()
 {
 #ifdef WIN32
@@ -307,16 +456,16 @@ SocketPort::~SocketPort()
 	endSocket();
 }
 
-void SocketPort::attach(SocketService *svc)
+void SocketPort::attach( SocketService* svc )
 {
 	if(service)
 		service->detach(this);
-	service = svc;
+	service = svc;	
 	if(svc)
 		svc->attach(this);
 }
 
-Socket::Error SocketPort::connect(const InetAddress &ia, tpport_t port)
+Socket::Error SocketPort::connect(const IPV4Address &ia, tpport_t port)
 {
 	struct sockaddr_in addr;
 	Error rtn = errSuccess;
@@ -333,6 +482,9 @@ Socket::Error SocketPort::connect(const InetAddress &ia, tpport_t port)
 	u_long opts = 1;
 	ioctlsocket(so,FIONBIO,&opts);
 #endif
+	// Win32 will crash if you try to connect to INADDR_ANY.
+	if ( INADDR_ANY == addr.sin_addr.s_addr )
+	        addr.sin_addr.s_addr = INADDR_LOOPBACK;
 	if(::connect(so, (struct sockaddr *)&addr, (socklen_t)sizeof(addr)))
 		rtn = connectError();
 #ifndef WIN32
@@ -344,26 +496,59 @@ Socket::Error SocketPort::connect(const InetAddress &ia, tpport_t port)
 	return rtn;
 }
 
+#ifdef  CCXX_IPV6
+Socket::Error SocketPort::connect(const IPV6Address &ia, tpport_t port)
+{
+	struct sockaddr_in6 addr;
+	Error rtn = errSuccess;
+
+	memset(&addr, 0, sizeof(addr));
+	addr.sin6_family = AF_INET6;
+	addr.sin6_addr = getaddress(ia);
+	addr.sin6_port = htons(port);
+
+#ifndef WIN32
+	long opts = fcntl(so, F_GETFL);
+	fcntl(so, F_SETFL, opts | O_NDELAY);
+#else
+	u_long opts = 1;
+	ioctlsocket(so,FIONBIO,&opts);
+#endif
+	// Win32 will crash if you try to connect to INADDR_ANY.
+	if(!memcmp(&addr.sin6_addr, &in6addr_any, sizeof(in6addr_any)))
+		memcpy(&addr.sin6_addr, &in6addr_loopback, sizeof(in6addr_loopback));
+	if(::connect(so, (struct sockaddr *)&addr, (socklen_t)sizeof(addr)))
+		rtn = connectError();
+#ifndef WIN32
+	fcntl(so, F_SETFL, opts);
+#else
+	opts = 0;
+	ioctlsocket(so,FIONBIO,&opts);
+#endif
+	return rtn;
+}
+
+#endif
 
 void SocketPort::setTimer(timeout_t ptimer)
 {
 	TimerPort::setTimer(ptimer);
-	if(service)
-		service->update();
+  if( service )
+  	service->update();
 }
 
 void SocketPort::incTimer(timeout_t ptimer)
 {
 	TimerPort::incTimer(ptimer);
-	if(service)
-		service->update();
+  if( service )
+  	service->update();
 }
 
 void SocketPort::setDetectPending( bool val )
 {
 	if ( detect_pending != val ) {
 		detect_pending = val;
-#ifdef CCXX_USE_POLL
+#ifdef USE_POLL
 		if ( ufd ) {
 			if ( val ) {
 				ufd->events |= POLLIN;
@@ -372,8 +557,8 @@ void SocketPort::setDetectPending( bool val )
 			}
 		}
 #endif
-		if(service)
-			service->update();
+    if( service )
+  		service->update();
 	}
 }
 
@@ -381,7 +566,7 @@ void SocketPort::setDetectOutput( bool val )
 {
 	if ( detect_output != val ) {
 		detect_output = val;
-#ifdef CCXX_USE_POLL
+#ifdef USE_POLL
 		if ( ufd ) {
 			if ( val ) {
 				ufd->events |= POLLOUT;
@@ -390,8 +575,8 @@ void SocketPort::setDetectOutput( bool val )
 			}
 		}
 #endif
-		if(service)
-			service->update();
+    if( service )
+  		service->update();
 	}
 }
 
@@ -439,8 +624,8 @@ private:
 };
 #endif
 
-SocketService::SocketService(int pri) :
-Thread(pri), Mutex()
+SocketService::SocketService(int pri, size_t stack, const char *id) :
+Thread(pri, stack), Mutex(id)
 {
 	first = last = NULL;
 	count = 0;
@@ -476,7 +661,7 @@ SocketService::~SocketService()
 void SocketService::attach(SocketPort *port)
 {
 	enterMutex();
-#ifdef	CCXX_USE_POLL
+#ifdef	USE_POLL
 	port->ufd = 0;
 #endif
 	if(last)
@@ -511,7 +696,7 @@ void SocketService::attach(SocketPort *port)
 void SocketService::detach(SocketPort *port)
 {
 	enterMutex();
-#if !defined(CCXX_USE_POLL) && !defined(WIN32)
+#if !defined(USE_POLL) && !defined(WIN32)
 	FD_CLR(port->so, &connect);
 #endif
 	if(port->prev) {
@@ -553,7 +738,7 @@ void SocketService::run(void)
 	unsigned char buf;
 
 #ifndef WIN32
-#ifdef 	CCXX_USE_POLL
+#ifdef 	USE_POLL
 	
 	Poller			  mfd;
 	pollfd			* p_ufd;
@@ -617,7 +802,7 @@ void SocketService::run(void)
 		}
 
 #ifndef WIN32
-#ifdef	CCXX_USE_POLL
+#ifdef	USE_POLL
 
 		bool	reallocate = false;
 		
@@ -870,7 +1055,7 @@ retry:
 }						
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif		
 	
 /** EMACS **
