@@ -80,6 +80,12 @@ String::String()
 	init();
 }
 
+String::String(std::string str)
+{
+       init();
+       set(str.c_str());
+}
+
 String::String(const String &original)
 {
 	init();
