@@ -39,8 +39,17 @@
 // If you do not wish that, delete this exception notice.
 
 #include <cc++/config.h>
+#ifdef	CCXX_WITHOUT_EXTRAS
 #include <cc++/export.h>
-#include <cc++/urlstring.h>
+#endif
+#include <cc++/exception.h>
+#include <cc++/thread.h>
+#include <cc++/address.h>
+#include <cc++/socket.h>
+#ifndef	CCXX_WITHOUT_EXTRAS
+#include <cc++/export.h>
+#endif
+#include <cc++/url.h>
 
 #include <string>
 #include <cstdio>
@@ -235,7 +244,7 @@ CCXX_EXPORT(char*) urlEncode(const char *source, char *dest, unsigned max)
 		ch = (unsigned char)*source;
 		if(*source == ' ')
 			*(dest++) = '+';
-		else if(isalnum(*source) || strchr("/.-", *source))
+		else if(isalnum(*source) || strchr("/.-:;,", *source))
 			*(dest++) = *source;
 		else
 		{
@@ -250,22 +259,22 @@ CCXX_EXPORT(char*) urlEncode(const char *source, char *dest, unsigned max)
 	return ret;
 }
 
+#if defined(DYNAMIC_LOCAL_ARRAYS)
+
 /** @relates URLStream
  * Encode a STL string using base64 coding into a STL string
  * @return base 64 encoded string
  * @param src source string
  */
-CCXX_EXPORT(std::string) b64Encode(const std::string& src)
+CCXX_EXPORT(String) b64Encode(const String& src)
 {
   size_t limit = (src.length()+2)/3*4+1;  // size + null must be included
-  char* buffer = new char[limit];
+  char buffer[limit];
 
   unsigned size = b64Encode((const unsigned char *)src.c_str(), src.length(), buffer, limit);
   buffer[size] = '\0';
 
-  std::string final = string(buffer);
-  delete buffer;
-  return final;
+  return String(buffer);
 }
 
 /** @relates URLStream
@@ -275,17 +284,15 @@ CCXX_EXPORT(std::string) b64Encode(const std::string& src)
  * @return decoded string
  * @param src     source string
  */
-CCXX_EXPORT(std::string) b64Decode(const std::string& src)
+CCXX_EXPORT(String) b64Decode(const String& src)
 {
   size_t limit = src.length()/4*3;
-  unsigned char* buffer = new unsigned char[limit+1];
+  unsigned char buffer[limit+1];
 
   unsigned size = b64Decode(src.c_str(), buffer, limit);
   buffer[size] = '\0';
 
-  std::string final = string((char*)buffer);
-  delete buffer;
-  return final;
+  return String((char *)buffer);
 }
 
 /** @relates URLStream
@@ -294,17 +301,15 @@ CCXX_EXPORT(std::string) b64Decode(const std::string& src)
  * @param src     source buffer
  * @param srcsize source buffer size
  */
-CCXX_EXPORT(std::string) b64Encode(const unsigned char *src, size_t srcsize)
+CCXX_EXPORT(String) b64Encode(const unsigned char *src, size_t srcsize)
 {
   size_t limit = (srcsize+2)/3*4+1;
-  char* buffer = new char[limit];
+  char buffer[limit];
 
   unsigned size = b64Encode(src, srcsize, buffer, limit);
   buffer[size] = '\0';
 
-  std::string final = string(buffer);
-  delete buffer;
-  return final;
+  return String(buffer);
 }
 
 /** @relates URLStream
@@ -316,12 +321,13 @@ CCXX_EXPORT(std::string) b64Encode(const unsigned char *src, size_t srcsize)
  * @param dst     destination octet buffer
  * @param dstsize destination buffer size
  */
-CCXX_EXPORT(size_t) b64Decode(const std::string& src,
+CCXX_EXPORT(size_t) b64Decode(const String& src,
   unsigned char *dst, size_t dstsize)
 {
   return b64Decode(src.c_str(), dst, dstsize);
 }
+#endif
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
